package File::Rewrite;

use strict;
use warnings;

use autodie ':all';
use Exporter 'import';

our @EXPORT_OK = qw(fileset);

sub fileset {
	my ($filename, $replace, $search) = @_;
	$search = $replace if not defined $search;
	my $tmpname = "$filename.$$.tmp";
	my $changes = 0;

	open my $src,  '<', $filename;
	open my $dest, '>', $tmpname;
	while (readline $src) {
		chomp;
		if ($_ =~ (ref $search ? $search : qr/^\Q$search\E$/)) {
			if (defined $replace and $_ eq $replace) {
				undef $replace;
			}
			else {
				$changes++;
				next;
			}
		}
		print {$dest} $_, $/;
	}
	close $src;
	if (defined $replace) {
		$changes++;
		print {$dest} $replace, $/;
	}
	close $dest;

	if ($changes) {
		rename $tmpname, $filename;
	}
	else {
		unlink $tmpname;
	}
	return $changes;
}

1;

=head1 NAME

File::Rewrite

=head1 SYNOPSIS

	use File::Rewrite 'fileset';
	fileset('/tmp/somefile', undef, qr/match/);

=head1 DESCRIPTION

=head1 AUTHOR

Mischa POSLAWSKY <perl@shiar.org>

=head1 LICENSE

Copyright. All rights reserved.

